@@ -26,162 +26,8 @@ import math
 import collections
 from pyspatiotemporalgeom.utilities import convexHull
 import pyspatiotemporalgeom.region as region
+from pyspatiotemporalgeom.utilities import triangleLibrary
 
-def dot( u, v ):
-        ''' 
-        returns dot product of vectors u and v
-        
-        u and v are 3-tuples representing a 3D point: ``(x,y,z)``
-
-        This function is used to compute triangle/triangle intersections
-
-        Returns a number
-        '''
-        return u[0]*v[0] + u[1]*v[1] + u[2]*v[2]
-
-def crossProduct( v, u ):
-        '''
-        returns cross product of u and v
-
-        u and v are 3-tuples representing a 3D point: ``(x,y,z)``
-
-        This function is used to compute triangle/triangle intersections
-
-        Returns a 3-tuple of numbers (a 3D point): ``(x,y,z)``
-        '''
-        return( (v[1]*u[2]-u[1]*v[2]), (-v[0]*u[2]+u[0]*v[2]),(v[0]*u[1]-u[0]*v[1]) )
-
-
-def triangleTriangleIntersection( tri1, tri2 ):
-        '''
-        Input:  two triangles of the form: ``((x1,y1,z1),(x2,y2,z2),(x3,y3,z3))``
-
-        triangles always have points [0] and [1] from the same slice. In other words, for the representation listed above, z1 = z2 ALWAYS. 
-        
-        So ... always make segs with [0][2] and [1][2] for tri tri intersection
-        
-        We are only interested in intersections in the interior of triangles.
-
-        RETURNS:
-
-          ``True`` if the triangles intersect in their interior
-
-          ``False`` if the trinagle's interiors do not intersect.  
-
-          Triangles with coincident edges but disjoint interiors return ``False``
-        '''
-        t1s1 = (tri1[0], tri1[2] )
-        t1s2 = (tri1[1], tri1[2] )
-        t2s1 = (tri2[0], tri2[2] )
-        t2s2 = (tri2[1], tri2[2] )
-        #for i in range(2):
-        if triangleRayIntersection(   t1s1, tri2 ) > 0:
-                return True
-        elif triangleRayIntersection( t1s2, tri2 ) > 0:
-                return True
-        elif triangleRayIntersection( t2s1, tri1 ) > 0:
-                return True
-        elif triangleRayIntersection( t2s2, tri1 ) > 0:
-                return True
-        #       tri1 = (tri1[1], tri1[0], tri1[2] )
-        #       tri2 = (tri2[1], tri2[0], tri2[2] )
-        return False
-
-
-def triangleRayIntersection( ray, tri ):
-        '''
-        
-        Test if a triangle and a ray intersect.  A ray is actually a seg in our case. 
-        
-        To conver this algorithm to use rays, change the line:
-            
-        ``if r <= 0.0 or r >= 1.0:``
-        
-        to:
-        
-        ``if r <= 0.0:``
-
-        We will always pass segs, since triangles will end at slice boundaries
-        
-        tri is a triangle a tuple of 3, 3D points.  Triangle vertices in arbitrary order
-        
-        Returns:
-        
-        -1: triangle is degenerate (seg or a point)
-        
-        0: disjoint
-        
-        1: intersect in unique point
-        
-        2: intersects in the same plane. (the intersection is a line segment)  
-        
-        This function is modeled from triangle ray intersection algorithm at http://geomalgorithms.com/body_a06-_intersect-2.html#intersect3D_RayTriangle(),
-        although this algorithm is different to handle specific intersection cases differently.  Comments are included that identify
-        portions specific to our intersection needs
-        The code at http://geomalgorithms.com/body_a06-_intersect-2.html#intersect3D_RayTriangle() has the following copyright:
-        
-        .. code-block:: c
-
-          //Copyright 2001 softSurfer, 2012 Dan Sunday
-          // This code may be freely used and modified for any purpose
-          // providing that this copyright notice is included with it.
-          // SoftSurfer makes no warranty for this code, and cannot be held
-          // liable for any real or imagined damage resulting from its use.
-          // Users of this code must verify correctness for their application.
-        
-        '''
-        # easy test.  if the ray is a tri seg, return not intersecting
-        # we don't consider a ray that is identical to a triangle edge to be intersecting
-        tsegs = ((tri[0], tri[1] ), (tri[1], tri[0] ), (tri[0], tri[2] ), (tri[2], tri[0] ), (tri[1], tri[2] ), (tri[2], tri[1] ))
-        if ray in tsegs:
-                return 0
-
-        
-        SMALL_NUM = 0.00000001
-        #u v n will be triangle vectors
-        # dir, w0 and w will be ray vectors
-        # r, a, b will be params to calc ray-plane intersect
-        
-        # get triangle edge vects and normal to plane
-        u = (tri[1][0]-tri[0][0], tri[1][1]-tri[0][1], tri[1][2]-tri[0][2])
-        v = (tri[2][0]-tri[0][0], tri[2][1]-tri[0][1], tri[2][2]-tri[0][2])
-        n = crossProduct( u,v )
-        if n == (0,0,0):
-                return -1 #degenerate
-        dir = (ray[1][0]-ray[0][0], ray[1][1]-ray[0][1], ray[1][2]-ray[0][2]) #ray direction vect
-        w0 = (ray[0][0]-tri[0][0], ray[0][1]-tri[0][1], ray[0][2]-tri[0][2])
-        a = float( -1*dot( n, w0 ) )
-        b = float( dot( n, dir ) )
-        if math.fabs( b ) < SMALL_NUM: #ray is parallel to tri
-                if a == 0:  # ray lies in tri
-                        return 2  # ray overlaps the triangle
-                else:
-                        return 0 #ray disjoint from plane
-        # get intersect point of ray with triangle plane
-        r = a/float(b)
-        #check if intersect is beyond end of seg.  For a ray, only test with 0.0
-        # we use <= and >= becuase we don't care abount end point intersections
-        if r <= 0.0 or r >= 1.0: 
-                return 0
-        #get the intersection point of the seg and plane
-        I = ( ray[0][0]+r*dir[0], ray[0][1]+r*dir[1], ray[0][2]+r*dir[2]  )
-        #find if the point is inside the tri
-        uu = dot( u, u)
-        uv = dot( u, v)
-        vv = dot( v, v)
-        w = ( I[0]-tri[0][0], I[1]-tri[0][1], I[2]-tri[0][2] )
-        wu = dot( w, u)
-        wv = dot( w, v)
-        D = uv*uv - uu*vv
-        # get and test the parametric coords
-        s = (uv*wv - vv*wu)/float(D)
-        if s < 0.0 or s > 1.0: # I is outside tri
-                return 0
-        t = (uv*wu - uu*wv)/float(D)
-        if t < 0.0 or (s+t) > 1.0: # I is outside tri
-                return 0
-        return 1  # intersection!  I is in the tri
-        
 
 def prepRegionForInterpolation( hsegs ):
         '''
@@ -509,14 +355,14 @@ def interpolate(r1,r2, startTime, endTime, noTriIntersectionChecks = False ):
         for t in r1HullTris:
                 for i,tList in enumerate( r1ConcTris ):
                         for c in tList:
-                                if triangleTriangleIntersection( t,c ):
+                                if triangleLibrary.triangle3DTriangle3DIntersection( t,c ):
                                         intersectingConcs.append( ((i,1), (i,1) ) )
                                         break
                 
         for t in r2HullTris:
                 for i,tList in enumerate( r2ConcTris ):
                         for c in tList:
-                                if triangleTriangleIntersection( t,c ):
+                                if triangleLibrary.triangle3DTriangle3DIntersection( t,c ):
                                         intersectingConcs.append( ((i,2), (i,2) ) )
                                         break
                                         
@@ -530,7 +376,7 @@ def interpolate(r1,r2, startTime, endTime, noTriIntersectionChecks = False ):
                 for j in range( i+1, len( c1AllConcTris ) ):
                         c1 = c1AllConcTris[i]
                         c2 = c1AllConcTris[j]
-                        if triangleTriangleIntersection( c1[0], c2[0] ):
+                        if triangleLibrary.triangle3DTriangle3DIntersection( c1[0], c2[0] ):
                                 intersectingConcs.append( ((c1[1],c1[2]), (c2[1],c2[2]) ) )
         
 #               evapConcs = set(r1ConcsThatIntersectWithHullTris)
